# @summary Install Vagrant, Virtualbox used with shell executor for acceptance tests of Puppet modules.
#
#@param install_ruby
#  This boolean at true the module install Ruby on the runner box (used with shell executor during puppet testing).
#@param install_vagrant
#  This boolean at true the module install Vagrant on the runner box (used with shell executor during puppet testing, hypervisor=vagrant).
#@param install_virtualbox
#  This boolean at true the module install Virtualbox on the runner box (used with shell executor during puppet testing).
#@param runner_account
#  System user account granted to use Docker engine.
#@param vagrant_package_name
#  The Vagrant package name installed by shell helper.
#@param vagrant_package_version
#  The Vagrant package version installed by shell helper.
#@param virtualbox_package_version
#  The VirtualBox package version installed by shell helper.
#@param rbenv_sources
#  The URL used to get Rbenv.
#@param rbenv_tag
#  The tag of Rbenv installed by shell helper.
#@param rubybuild_sources
#  The URL used to get Ruby Build
#@param rubybuild_tag
#  The tag of Ruby Build installed by shell helper.
#@param ruby_versions
#  The list of Ruby versions installed by shell helper.
#
class gitlabrunner::helper::shell (
  Boolean $install_ruby,
  Boolean $install_vagrant,
  Boolean $install_virtualbox,
  String $runner_account,
  String[1] $vagrant_package_name,
  String[1] $vagrant_package_version,
  String[1] $virtualbox_package_version,
  String[1] $rbenv_sources,
  String[1] $rbenv_tag,
  String[1] $rubybuild_sources,
  String[1] $rubybuild_tag,
  Array[String[1]] $ruby_versions,
) {
  require gitlabrunner::install

  $_rbenv_root = "/home/${runner_account}/.rbenv"

  case $facts['os']['release']['major'] {
    '18.04','20.04': {
      $_pkg_update_cache = 'Exec[apt_update]'

      $_vagrant_package_url = "https://releases.hashicorp.com/vagrant/${vagrant_package_version}/${vagrant_package_name}_${vagrant_package_version}_x86_64.deb"
      $_vagrant_package_path = "/root/${vagrant_package_name}_${vagrant_package_version}_x86_64.deb"
      $_vagrant_cmd_install = "/usr/bin/dpkg -i ${_vagrant_package_path}"
      $_vagrant_cmd_install_check = "/usr/bin/apt-cache show ${vagrant_package_name} | grep '^Version:.*${vagrant_package_version}$'"

      $_shell_custom_profile = "/home/${runner_account}/.profile"
      $_ruby_requirement = [
        'libssl-dev',
        'libreadline-dev',
        'zlib1g-dev',
        'make',
        'gcc',
      ]
    }
    default: {
      fail("unmanaged os ${facts['os']['distro']['codename']}")
    }
  }

  if $install_vagrant {
    archive { 'download vagrant package' :
      ensure => present,
      path   => $_vagrant_package_path,
      source => $_vagrant_package_url,
      user   => 0,
      group  => 0,
    }
    -> exec { 'install vagrant' :
      command => $_vagrant_cmd_install,
      unless  => $_vagrant_cmd_install_check,
    }
  }

  if $install_virtualbox {
    # install virtualbox
    class { 'virtualbox':
      version => $virtualbox_package_version,
    }
  }

  if $install_ruby {
    # install ruby
    package { $_ruby_requirement :
      ensure  => present,
      require => $_pkg_update_cache,
    }

    file { $_shell_custom_profile:
      ensure  => file,
      owner   => $runner_account,
      group   => $runner_account,
      mode    => '0644',
      content => "PATH=/home/${runner_account}/.rbenv/bin:/home/${runner_account}/.rbenv/shims:\$PATH",
    }
    -> vcsrepo { $_rbenv_root :
      ensure   => present,
      provider => 'git',
      owner    => $runner_account,
      group    => $runner_account,
      source   => $rbenv_sources,
      revision => $rbenv_tag,
    }
    -> file { ["${_rbenv_root}/plugins", "${_rbenv_root}/cache"]:
      ensure => directory,
      owner  => $runner_account,
      group  => $runner_account,
      mode   => '0755',
    }
    -> vcsrepo { "${_rbenv_root}/plugins/ruby-build" :
      ensure   => present,
      provider => 'git',
      owner    => $runner_account,
      group    => $runner_account,
      source   => $rubybuild_sources,
      revision => $rubybuild_tag,
    }

    $ruby_versions.each | String $_ruby_version | {
      exec { "rbenv install ${_ruby_version}" :
        user        => $runner_account,
        creates     => "${_rbenv_root}/versions/${_ruby_version}/bin/ruby",
        path        => "${_rbenv_root}/bin:${_rbenv_root}/shims:/bin:/usr/bin",
        cwd         => "/home/${runner_account}",
        timeout     => 600,
        environment => [
          "HOME=/home/${runner_account}",
        ],
        require     => [
          Package[$_ruby_requirement],
          Vcsrepo[$_rbenv_root],
          Vcsrepo["${_rbenv_root}/plugins/ruby-build"],
        ],
      }
      -> exec { "ruby ${_ruby_version} gem install bundler" :
        user        => $runner_account,
        command     => 'gem install bundler',
        creates     => "${_rbenv_root}/versions/${_ruby_version}/bin/bundle",
        path        => "${_rbenv_root}/bin:${_rbenv_root}/shims:/bin:/usr/bin",
        cwd         => "/home/${runner_account}",
        environment => [
          "HOME=/home/${runner_account}",
          "RBENV_VERSION=${_ruby_version}",
        ],
      }
    }
  }
}
