# @summary Install gitlab runner.
#
# @example
#   include gitlabrunner::install
#
# @api private
#
class gitlabrunner::install (
  String[1] $ensure = $gitlabrunner::ensure,
) {
  assert_private()

  require apt

  # version of gitlab-runner is hard coded here because we use output of
  # command "/usr/bin/gitlab-runner list" used to forge a custom fact.
  # The output of this command can change with versions of gitlab-runner.
  # This can break the parsing of output and so break the custom fact.
  $_package_name = 'gitlab-runner'
  $_package_version = $ensure
  $_gitlab_source_installer_url = 'https://packages.gitlab.com/install/repositories/runner/gitlab-runner/script.deb.sh'
  $_gitlab_source_installer_path = '/usr/local/sbin/gitlab_source_installer.sh'
  $_gitlab_source_installer_creates = '/etc/apt/sources.list.d/runner_gitlab-runner.list'

  # Gitlab source installer requirements
  $_required_packages = [
    'curl',
    'gnupg',
  ]

  ensure_packages($_required_packages)

  archive { 'download gitlab source installer' :
    ensure => present,
    path   => $_gitlab_source_installer_path,
    source => $_gitlab_source_installer_url,
    user   => 0,
    group  => 0,
  }
  -> file { $_gitlab_source_installer_path :
    ensure => file,
    owner  => 0,
    group  => 0,
    mode   => '0700',
    notify => Exec[$_gitlab_source_installer_path],
  }

  exec { $_gitlab_source_installer_path :
    user        => 'root',
    refreshonly => true,
    require     => Package[$_required_packages],
    creates     => $_gitlab_source_installer_creates,
  }

  apt::pin { $_package_name :
    ensure      => present,
    explanation => 'Prefer GitLab provided packages over the Debian native one',
    packages    => $_package_name,
    origin      => 'packages.gitlab.com',
    priority    => 1001,
  }

  apt::key { 'GitLab package repository signing key':
    id     => $gitlabrunner::repository_key_id,
    source => $gitlabrunner::repository_key_url,
  }

  package { $_package_name :
    ensure  => $_package_version,
    require => [
      Apt::Pin[$_package_name],
      Apt::Key['GitLab package repository signing key'],
      Exec[$_gitlab_source_installer_path],
    ],
    notify  => Service['gitlab-runner'],
  }

  service { 'gitlab-runner' :
    ensure  => running,
    start   => '/usr/bin/gitlab-runner start',
    stop    => '/usr/bin/gitlab-runner stop',
    status  => '/usr/bin/gitlab-runner status',
    require => Package[$_package_name],
  }
}
